# Copyright 2018, Qualita Seguranca e Saude Ocupacional. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
import os

import esocial
from esocial import client

here = os.path.dirname(os.path.abspath(__file__))
# there = os.path.dirname(os.path.abspath(esocial.__file__))

def ws_factory():
    employer_id = {
        'tpInsc': 1,
        'nrInsc': '12345678901234'
    }
    return client.WSClient(
        pfx_file=os.path.join(here, 'certs', 'libesocial-cert-test.pfx'),
        pfx_passw='cert@test',
        employer_id=employer_id,
        target=2
    )
